MODULE matrix;
(*******************************************************************
   Errechnung eines magischen Quadrates.
   Summe der Horizontalen = Summe der Vertikalen = Summe der
   Diagonalen im Quadrat
    z.B.                 6  1  8
                         7  5  3
                         2  9  4
 *******************************************************************)
FROM InOut IMPORT WriteString,
                  WriteLn,
                  WriteCard,
                  ReadCard;

CONST Nmax = 15;  (* maximales n *)

TYPE mat = ARRAY[1..Nmax], [1..Nmax] OF CARDINAL;

VAR
  a : mat;       (* die besagte Matrix *)
  n : CARDINAL;  (* Breite, Hoehe der Matrix *)


(*******************************************************************)
(* Procedure LOESCHEN                                              *)
(*******************************************************************)
(* Setzt alle Werte der Matrix auf 0                               *)
(*******************************************************************)
PROCEDURE loeschen(VAR a : mat);
VAR
  i, j : CARDINAL;  (* Laufvariablen *)

BEGIN
  FOR i:=1 TO Nmax DO
    FOR j:=1 TO Nmax DO
      a[i,j]:=0;
    END;
  END;
END loeschen;


(*******************************************************************)
(* Procedure ALLESVOLL                                             *)
(*******************************************************************)
(* Ueberprueft, ob alle Stellen der Matrix belegt sind.            *)
(*******************************************************************)
PROCEDURE allesvoll(n : CARDINAL; VAR a : mat) : BOOLEAN;
VAR
  i, j : CARDINAL;    (* Laufvariablen *)
  b : BOOLEAN;

BEGIN
  FOR i:=1 TO n DO
    FOR j:=1 TO n DO
      IF (a[i,j]=0) THEN
        b:=FALSE;
        RETURN b;
      END;
    END;
  END;
  b:=TRUE;
  RETURN b;
END allesvoll;


(*******************************************************************)
(* Procedure INCINDEX                                              *)
(*******************************************************************)
(* Erhoeht die Laufvariable i um 1.                                *)
(* Falls i>n dann wird i auf 1 gesetzt                             *)
(*******************************************************************)
PROCEDURE incindex(i, n : CARDINAL) : CARDINAL;
BEGIN
  INC(i);
  IF (i>n) THEN RETURN 1
           ELSE RETURN i; END;
END incindex;


(*******************************************************************)
(* Procedure DECINDEX                                              *)
(*******************************************************************)
(* Erniedrigt die Laufvariable i um 1.                             *)
(* Falls i<1 dann wird i auf n gesetzt.                            *)
(*******************************************************************)
PROCEDURE decindex(i, n : CARDINAL) : CARDINAL;
BEGIN
  DEC(i);
  IF (i<1) THEN RETURN n
           ELSE RETURN i; END;
END decindex;


(*******************************************************************)
(* Procedure MAKEMAGIC                                             *)
(*******************************************************************)
(* generiert das magische Quadrat der Groesse n x n                *)
(*******************************************************************)
PROCEDURE makemagic(n : CARDINAL; VAR a : mat);
VAR
  ialt, jalt, ineu, jneu,  (* Laufvariablen *)
  k : CARDINAL;

BEGIN
  k:=1;
  ialt:=1;
  jalt:=(n DIV 2)+1;
  a[ialt,jalt]:=k;
  INC(k);
  REPEAT
    ineu:=decindex(ialt, n);
    jneu:=decindex(jalt, n);

    IF (a[ineu,jneu]=0) THEN
      a[ineu,jneu]:=k;
      INC(k);
    ELSE
      ineu:=incindex(ialt, n);
      jneu:=jalt;
      a[ineu,jneu]:=k;
      INC(k);
    END;
    ialt:=ineu; jalt:=jneu;
  UNTIL allesvoll(n, a);
END makemagic;


(*******************************************************************)
(* Procedure OUTMAGIC                                              *)
(*******************************************************************)
(* gibt das magische Quadrat aus                                   *)
(*******************************************************************)
PROCEDURE outmagic(n : CARDINAL; VAR a: mat);
VAR
  i, j : CARDINAL;   (* Laufvariablen *)

BEGIN
  FOR i:=1 TO n DO
    FOR j:=1 TO n DO
      WriteCard(a[i,j], 5);
    END;
    WriteLn;
  END;
END outmagic;


(*******************************************************************)
(* Hauptprogramm                                                   *)
(*******************************************************************)
BEGIN
  (* Nachricht ausgeben *)
  WriteLn; WriteLn;
  WriteString('Dieses Programm berechnet ein magisches Quadrat');
  WriteString(' der Groesse n, 1<n<=15');
  WriteLn; WriteLn;
  loeschen(a);  (* Matrix loeschen *)
  REPEAT
    WriteString('Bitte n eingeben (n ungerade):  '); (* n einlesen *)
    ReadCard(n);
    WriteLn;
  UNTIL (n MOD 2=1) AND (n>1) AND (n<=15);
  WriteLn;
  makemagic(n, a);  (* Matrix generieren und *)
  outmagic(n, a);   (* ausgeben *)
END matrix.
