MODULE poker;
(*******************************************************************
  Zahlenpoker.
  Nach Eingabe einer 5-stelligen Zahl gibt der Computer aus, ob es
  sich um einen Pasch, Doppelpasch, Drilling usw. handelt.
  z. B.: Eingabe:   "41523"
         Ausgabe: "Dies ist eine Strae"
 ********************************************************************)

FROM InOut IMPORT ReadString,
                  WriteString,
                  WriteLn,
                  Read;
FROM Strings IMPORT Length,
                    Concat,
                    Pos;
FROM Str IMPORT StrToCard;

TYPE
  string5 = ARRAY[1..5] OF CHAR;

VAR
  s : string5; (* der einzugebende String *)
  ch : CHAR;


(*******************************************************************)
(* Procedure EINGEBEN                                              *)
(*******************************************************************)
(* Liest solange st : string5 ein, bis gilt:                       *)
(* st ist eine Zahl zwischen 00001 und 99999.                      *)
(*******************************************************************)
PROCEDURE eingeben(VAR st : string5);
VAR
  zahl : LONGCARD;
  done : BOOLEAN;

BEGIN
  REPEAT
    WriteString('Bitte eine fuenfstellige Zahl eingeben: ');
    ReadString(st);
    WriteLn;
    zahl:=StrToCard(st, 10, done);
  UNTIL (Length(st)=5) AND done;
  WriteLn;
END eingeben;


(*******************************************************************)
(* Procedure SORTIEREN                                             *)
(*******************************************************************)
(* Sortiert die Ziffern von st : string5 der Groesse nach.         *)
(* z. B.: 34152 => 12345.                                          *)
(*******************************************************************)
PROCEDURE sortieren(VAR st : string5);
VAR
  hilfchar : CHAR;
  i, j : CARDINAL;

BEGIN
  FOR i:=1 TO 5 DO
    FOR j:=i+1 TO 5 DO
      IF (ORD(st[i])>ORD(st[j])) THEN
        hilfchar:=st[i];
        st[i]:=st[j];
        st[j]:=hilfchar;
      END;
    END;
  END;
END sortieren;


(*******************************************************************)
(* Procedure TUPEL                                                 *)
(*******************************************************************)
(* Ueberprueft, ob in st : string5 ein Paar (anzahl=2), ein        *)
(* Trilling (anzahl=3) oder ein Vierer (anzahl=4) vorhanden ist.   *)
(*******************************************************************)
PROCEDURE tupel(st : string5; anzahl : CARDINAL) : BOOLEAN;
VAR
  teilst : string5;
  j, i : CARDINAL;

BEGIN
  IF (anzahl<=1) OR (anzahl>5) THEN
    RETURN FALSE;
  END;
  FOR j:=0 TO 9 DO
    FOR i:=1 TO 5 DO teilst[i]:=CHR(0); END;
    FOR i:=1 TO anzahl DO
      teilst[i]:=CHR(ORD('0')+j);
    END;
    IF (Pos(teilst, st)<Length(s)) THEN
      RETURN TRUE;
    END;
  END;
  RETURN FALSE;
END tupel;


(*******************************************************************)
(* Procedure DOPPELPAAR                                            *)
(*******************************************************************)
(* Prueft, ob st : string5 ein Doppelpaar enthaelt.                *)
(*******************************************************************)
PROCEDURE doppelpaar(st : string5) : BOOLEAN;
VAR
  i, j, position : CARDINAL;
  teilst : string5;

BEGIN
  IF tupel(st, 2) THEN
    FOR i:=1 TO 5 DO teilst[i]:=CHR(0); END;
    j:=0;
    LOOP
      FOR i:=1 TO 2 DO
        teilst[i]:=CHR(ORD('0')+j);
      END;
      position:=Pos(teilst, st);
      IF (position<Length(st)) THEN
        EXIT;
      END;
      INC(j);
    END;
    INC(position);
    FOR j:=position TO position+1 DO
      st[j]:='D';
    END;
    IF tupel(st, 2) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  ELSE
    RETURN FALSE;
  END;
END doppelpaar;


(*******************************************************************)
(* Procedure VOLLESHAUS                                            *)
(*******************************************************************)
(* Prueft, ob st : string5 ein Volles Haus darstellt.              *)
(*******************************************************************)
PROCEDURE volleshaus(st : string5) : BOOLEAN;
VAR
  i, j, position : CARDINAL;
  teilst : string5;

BEGIN
  IF tupel(st, 3) THEN
    FOR i:=1 TO 5 DO teilst[i]:=CHR(0); END;
    j:=0;
    LOOP
      FOR i:=1 TO 3 DO
        teilst[i]:=CHR(ORD('0')+j);
      END;
      position:=Pos(teilst, st);
      IF (position<Length(st)) THEN
        EXIT;
      END;
      INC(j);
    END;
    INC(position);
    FOR j:=position TO position+2 DO
      st[j]:='D';
    END;
    IF tupel(st, 2) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END;
  ELSE
    RETURN FALSE;
  END;
END volleshaus;


(*******************************************************************)
(* Procedure STRASSE                                               *)
(*******************************************************************)
(* Prueft, ob st : string5 eine Strasse darstellt.                 *)
(*******************************************************************)
PROCEDURE strasse(st : string5) : BOOLEAN;
VAR
  j : CARDINAL;

BEGIN
  FOR j:=2 TO 5 DO
    IF (ORD(st[j])<>ORD(st[1])+j-1) THEN
      RETURN FALSE;
    END;
  END;
  RETURN TRUE;
END strasse;


(*******************************************************************)
(* Procedure AUSWERTEN                                             *)
(*******************************************************************)
(* Wertet st : string5 aus und gibt das Ergebnis der Auswertung aus*)
(*******************************************************************)
PROCEDURE auswerten(st : string5);
BEGIN
  WriteString(st);
  IF tupel(st, 4) THEN
    WriteString(' ist ein Vierer!');
  ELSIF volleshaus(st) THEN
    WriteString(' ist ein Volles Haus!');
  ELSIF strasse(st) THEN
    WriteString(' ist eine Strasse!');
  ELSIF tupel(st, 3) THEN
    WriteString(' ist ein Trilling!');
  ELSIF doppelpaar(st) THEN
    WriteString(' ist ein Doppelpaar!');
  ELSIF tupel(st, 2) THEN
    WriteString(' ist ein Paar!');
  ELSE
    WriteString(' ist leider gar nichts!');
  END;
  WriteLn;
END auswerten;


(*******************************************************************)
(* Hauptprogramm                                                   *)
(*******************************************************************)
BEGIN
  WriteString('******************************************'); WriteLn;
  WriteString('*                                        *'); WriteLn;
  WriteString('*         Z A H L E N P O K E R          *'); WriteLn;
  WriteString('*                                        *'); WriteLn;
  WriteString('******************************************'); WriteLn;
  WriteLn;
  REPEAT
    eingeben(s);
    sortieren(s);
    auswerten(s);
    WriteLn;
    WriteString('Noch eine Zahl eingeben (j/n) ?');
    Read(ch);
    WriteString(ch);
    WriteLn; WriteLn;
  UNTIL (ch='N') OR (ch='n');
END poker.


