MODULE salesman;
(********************************)
(* Problem des Handelsreisenden *)
(********************************)
FROM InOut IMPORT ReadString,
                  WriteString,
                  WriteLn,
                  Read,
                  Write,
                  ReadCard,
                  WriteCard;
FROM Str   IMPORT Caps; (* Caps('test') => 'TEST' *)

CONST
  Maxstadt = 30;        (* maximale Anzahl der Staedte *)

TYPE
  matrix = ARRAY[1..Maxstadt],[1..Maxstadt] OF BOOLEAN;
  string20 = ARRAY[1..20] OF CHAR;
  string20feld = ARRAY[1..Maxstadt] OF string20;
  boolfeld = ARRAY[1..Maxstadt] OF BOOLEAN;
  cardfeld = ARRAY[1..Maxstadt] OF CARDINAL;

VAR
  verbindung : matrix;       (* beinhaltet die einzelnen Verbindungen *)
  stadtname : string20feld;  (* beinhaltet die Staedtenamen *)
  reiseroute : cardfeld;     (* beinhaltet die Reiseroute *)
  routenzeiger : CARDINAL;   (* Zeiger fuer das Feld reiseroute *)
  gewesen : boolfeld;        (* Reiseprotokoll (wo war man bereits?) *)
  anzstadt : CARDINAL;       (* Anzahl der Staedte *)
  routengefunden : CARDINAL; (* Anzahl der gefunden Routen *)


(*******************************************************************)
(* Procedure INIT                                                  *)
(*******************************************************************)
(* Initialisiert die Variablen und Felder des Programms und gibt   *)
(* eine Nachricht aus.                                             *)
(*******************************************************************)
PROCEDURE init;
VAR
  j, i : CARDINAL;              (* Laufvariablen *)

BEGIN
  FOR j:=1 TO Maxstadt DO       (* verbindung initialisieren  *)
    FOR i:=1 TO Maxstadt DO     (* = alle Elemente des Feldes *)
      verbindung[j, i]:=FALSE;  (* auf FALSE setzen. *)
    END;
  END;

  FOR j:=1 TO Maxstadt DO       (* stadtname und *)
    reiseroute[j]:=0;           (* reiseroute initialisieren  *)
    stadtname[j]:='';           (* alle Elemente = '' setzen *)
    gewesen[j]:=FALSE;          (* Reiseprotokoll zuruecksetzen *)
  END;

  anzstadt:=0;                  (* keine Anzahl der Staedte bekannt *)
  routengefunden:=0;            (* noch keine Reiseroute gefunden *)
  gewesen[1]:=TRUE;             (* Bei der ersten Stadt wird begonnen *)
  reiseroute[1]:=1;
  routenzeiger:=1;

  WriteLn; WriteLn;
  WriteString('Routenplanung fuer Handelsreisende'); WriteLn;
  WriteString('----------------------------------'); WriteLn;
  WriteLn;
END init;


(*******************************************************************)
(* Procedure lesestadtnamen                                        *)
(*******************************************************************)
(* Liest die Namen von anz Staedten ein und legt sie in der zu     *)
(* uebergebenden Variablen liste : string20feld ab.                *)
(*******************************************************************)
PROCEDURE lesestadtname(VAR liste : string20feld; anz : CARDINAL);
VAR
  j : CARDINAL;                     (* Laufvariable *)

BEGIN
  FOR j:=1 TO anz DO
    WriteString('Name der ');
    WriteCard(j, 0);
    WriteString('. Stadt: ');
    ReadString(liste[j]); WriteLn;  (* Wie heisst Stadt Nr. j ??? *)
  END;
  WriteLn;
END lesestadtname;


(*******************************************************************)
(* Procedure LESEVERBINDUNGEN                                      *)
(*******************************************************************)
(* Liest die Verbindungen der zwischen den verschiedenen Staedten  *)
(* ein und traegt sie in das zu uebergebende Feld plan : matrix    *)
(* ein.                                                            *)
(* liste ist die Liste der Staedtenamen, anz die Anzahl der Staedte*)
(*******************************************************************)
PROCEDURE leseverbindungen(VAR plan : matrix; liste : string20feld;
                                anz : CARDINAL);
VAR j, i : CARDINAL;    (* Laufvariablen *)
    taste : CHAR;       (* fuer die Tastatureingaben *)

BEGIN
  FOR j:=1 TO anz DO
    FOR i:=j+1 TO anz DO
       WriteString('Ist ');  WriteString(liste[j]);
       WriteString(' mit '); WriteString(liste[i]);
       WriteString(' verbunden (j/n) ? ');
       REPEAT
         Read(taste);
         Caps(taste);
       UNTIL (taste='J') OR (taste='N');     (* [J]a oder [N]ein *)
       Write(taste); WriteLn;
       IF (taste='J') THEN
          verbindung[i,j]:=TRUE;  (* Wenn i mit j verbunden, dann *)
          verbindung[j,i]:=TRUE;  (* ist auch j mit i verbunden.  *)
        END;  (* IF *)
    END;   (* FOR *)
    WriteLn;
  END;
END leseverbindungen;



(*******************************************************************)
(* Procedure AUSGABE                                               *)
(*******************************************************************)
(* gibt eine moegliche Reiseroute aus.                             *)
(*******************************************************************)
PROCEDURE ausgabe(VAR route : cardfeld; liste : string20feld;
                        anz : CARDINAL);
VAR j : CARDINAL;       (* Laufvariable *)
    ch : CHAR;          (* fuer Tastatureingaben *)

BEGIN
  INC(routengefunden);  (* eine Route mehr *)
  WriteString('Reiseroute Nr. ');
  WriteCard(routengefunden, 3);
  WriteString(' lautet: '); WriteLn;
  WriteString('--------------------------'); WriteLn;
  FOR j:=1 TO anz-1 DO
    WriteString(liste[route[j]]); WriteString(' --> ');
    WriteString(liste[route[j+1]]); WriteLn;
  END;
  WriteString(liste[route[anz]]); WriteString(' --> ');
  WriteString(liste[route[1]]); WriteLn;
  WriteLn;
  Read(ch);             (* nach jeder Route auf Tastendruck warten *)
END ausgabe;



(*******************************************************************)
(* Procedure FAHRPLAN                                              *)
(*******************************************************************)
(* berechnet die Reiseroute rekursiv.                              *)
(*******************************************************************)
PROCEDURE fahrplan(nummer, anz : CARDINAL);
VAR
  j,i : CARDINAL;        (* Laufvariablen *)
  alledurch : BOOLEAN;   (* =TRUE, wenn man ueberall war *)

BEGIN
  alledurch:=TRUE;
  FOR j:=1 TO anz DO     (* waren wir schon ueberall ??? *)
    IF (gewesen[j]=FALSE) THEN
      alledurch:=FALSE;
    END;
  END;
  IF alledurch THEN
    IF verbindung[1, reiseroute[anz]] THEN
      ausgabe(reiseroute, stadtname, anz);
    END;
  ELSE
    FOR j:=1 TO anz DO
      IF ((verbindung[nummer, j]=TRUE) AND (gewesen[j]=FALSE)) THEN
        INC(routenzeiger);
        reiseroute[routenzeiger]:=j;
        gewesen[j]:=TRUE;
        fahrplan(j, anz);
        gewesen[j]:=FALSE;
        DEC(routenzeiger);
      END;
    END;
  END;
END fahrplan;



(*******************************************************************)
(* Hauptprogramm                                                   *)
(*******************************************************************)
BEGIN
  init;
  REPEAT
    WriteString('Wieviele Staedte wollen Sie bereisen (max. ');
    WriteCard(Maxstadt, 0); WriteString('): ');
    ReadCard(anzstadt); WriteLn;
  UNTIL (anzstadt>0) AND (anzstadt<=Maxstadt);
  WriteLn; WriteLn;

  lesestadtname(stadtname, anzstadt);
  leseverbindungen(verbindung, stadtname, anzstadt);
  fahrplan(1, anzstadt);

  WriteLn;
  WriteString('Es wurden '); WriteCard(routengefunden, 0);
  WriteString(' Routen gefunden. '); WriteLn;
  WriteLn;
  IF (routengefunden>0) THEN
    WriteString('Die Reiseagentur wuenscht Ihnen eine gute Reise.');
    WriteLn;
  ELSE
    WriteString('Aufgrund der eingegebenen Daten sind wir');
    WriteString(' leider nicht in der Lage, '); WriteLn;
    WriteString('Ihnen einen Reiseplan zu erstellen.'); WriteLn;
  END;
  WriteLn; WriteLn;  (* und tschuess *)
END salesman.
