MODULE stube; (* Studentenbekanntschaft *)
(*******************************************************************
   Das Problem der Studentenbekanntschaften.
   z. B.: Max kennt Hans, Hans kennt Franz, somit kennt auch Max
          Franz.
   Der verwendete Algorithmus errechnet, welche Studenten sich auch
   ber mehrere andere Studenten kennen.
 *******************************************************************)
FROM InOut IMPORT ReadCard,
                  WriteCard,		                          			
                  WriteLn,
                  WriteString,
                  ReadString;
FROM Strings IMPORT CompareStr,
                    Assign;

CONST
  Maxstud = 100;  (* maximale Anzahl der zu bearbeitenden Studenten *)
  Minstud = 2;    (* minimale ------------------"------------------ *)

TYPE
  string10 = ARRAY[1..10] OF CHAR;
  matrix = ARRAY[1..Maxstud],[1..Maxstud] OF BOOLEAN;
  namenfeld = ARRAY[1..Maxstud] OF string10;

VAR
  name : namenfeld;    (* dieser ARRAY enthaelt die Namen der Stud. *)
  bekanntschaft : matrix;
  anzstud : CARDINAL;  (* Anzahl der Studenten *)


(*******************************************************************)
(* Procedure MATRIXINIT                                            *)
(*******************************************************************)
(* Initialisiert die Matrix fuer anz Studenten.                    *)
(*******************************************************************)
PROCEDURE matrixinit(VAR m : matrix; anz : CARDINAL);
VAR
  j , i : CARDINAL;

BEGIN
  FOR j:=1 TO anz DO
    FOR i:=1 TO anz DO
      m[j, i]:=FALSE;
    END;
    m[j,j]:=TRUE;  (* da sich jeder Student selber kennt *)
  END;
END matrixinit;


(*******************************************************************)
(* Procedure LESENAMEN                                             *)
(*******************************************************************)
(* Liest die Namen der einzelnen Studenten ein.                    *)
(*******************************************************************)
PROCEDURE lesenamen(VAR n : namenfeld; anz : CARDINAL);
VAR
  j : CARDINAL; (* Laufvariable *)

BEGIN
  WriteLn;
  FOR j:=1 TO anz DO
    WriteString('Bitte den Namen des ');
    WriteCard(j, 0);
    WriteString('. Studenten eingeben: ');
    ReadString(n[j]);
    WriteLn;
  END;
END lesenamen;


(*******************************************************************)
(* Procedure NUMMER                                                *)
(*******************************************************************)
(* Durchsucht das Feld n nach den Namen nam und gibt den Positions-*)
(* index zurueck.                                                  *)
(* Ist der Name nicht in n enthalten, so wird 0 zurueckgegeben.    *)
(*******************************************************************)
PROCEDURE nummer(n : namenfeld;
                 nam : string10;
                 anz : CARDINAL) : CARDINAL;
VAR
  j : CARDINAL; (* Laufvariable *)

BEGIN
  FOR j:=1 TO anz DO
    IF (CompareStr(n[j], nam)=0) THEN
      RETURN j;
    END;
  END;
  RETURN 0;
END nummer;


(*******************************************************************)
(* Procedure LESEBEKANNTSCHAFT                                     *)
(*******************************************************************)
(* Liest die Bekannten aller Studenten ein und erstellt die        *)
(* entsprechende Matrix.                                           *)
(*******************************************************************)
PROCEDURE lesebekanntschaft(VAR m : matrix; anz : CARDINAL);
VAR
  j, i : CARDINAL; (* Laufvariablen *)
  nam : string10;

BEGIN
  FOR j:=1 TO anz DO
    WriteLn;
    WriteString('Bitte geben Sie die Bekannten des Studenten ');
    WriteString(name[j]); WriteString(' ein: ');
    WriteLn;
    WriteString('Geben Sie * ein, um die Eingabe zu beenden.');
    WriteLn;
    REPEAT
      WriteString('Name: ');
      ReadString(nam);
      i:=nummer(name, nam, anz);
      IF (CompareStr(nam, '*')<>0) THEN
        IF (i>0) THEN
          WriteString('  =>  wird registriert!');
          m[j,i]:=TRUE;
          m[i,j]:=TRUE;
        ELSE
          WriteString('  ist mir unbekannt!');
        END;
      END;
      WriteLn;
    UNTIL (CompareStr(nam, '*')=0);
  END;
END lesebekanntschaft;


(*******************************************************************)
(* Procedure MATRIXBEWERTUNG                                       *)
(*******************************************************************)
(* Ueberprueft, wer wen kennt und legt das Ergebnis in der Matrix  *)
(* m ab.                                                           *)
(*******************************************************************)
PROCEDURE matrixbewertung(VAR m : matrix; anz : CARDINAL);
VAR
  i, j, k : CARDINAL;  (* Laufvariablen *)

BEGIN
  FOR j:=1 TO anz DO
    FOR i:=1 TO anz DO
      IF m[j, i] THEN
        FOR k:=i+1 TO anz DO
          IF m[j,k] THEN
            m[i,k]:=TRUE;
            m[k,i]:=TRUE;
          END;
        END;
      END;
    END;
  END;
END matrixbewertung;


(*******************************************************************)
(* Procedure MATRIXAUSGABE                                         *)
(*******************************************************************)
(* Gibt die Bekanntschaftsverhaeltnisse aus.                       *)
(*******************************************************************)
PROCEDURE matrixausgabe(m : matrix; anz : CARDINAL);
VAR
  i,j : CARDINAL; (* Laufvariablen *)
  allekennensich : BOOLEAN; (* ist TRUE, falls sich alle kennen *)
BEGIN
  WriteLn;
  allekennensich:=TRUE;
  FOR j:=1 TO anz DO
    FOR i:=1 TO anz DO
      IF NOT m[j,i] THEN
        WriteString(name[j]);
        WriteString(' kennt ');
        WriteString(name[i]);
        WriteString(' nicht!');
        WriteLn;
        allekennensich:=FALSE;
      END;
    END;
  END;
  IF allekennensich THEN
    WriteLn;
    WriteString('Alle Studenten kennen sich untereinander!!!');
    WriteLn; WriteLn;
  END;
END matrixausgabe;


(*******************************************************************)
(* Hauptprogramm                                                   *)
(*******************************************************************)
BEGIN
  WriteString('    STUDENTENBEKANNTSCHAFT');WriteLn;WriteLn;
  WriteString('Es soll geprft werden, ob sich mehrere Studenten auch');
  WriteLn;
  WriteString('ber mehrere Studenten kennen.');
  WriteLn;WriteLn;
  REPEAT
    WriteString('Bitte Anzahl der Studenten eingeben (min: ');
    WriteCard(Minstud, 0);
    WriteString(' /max: ');
    WriteCard(Maxstud, 0);
    WriteString('): ');
    ReadCard(anzstud);
    WriteLn;
  UNTIL (anzstud>=Minstud) AND (anzstud<=Maxstud);
  matrixinit(bekanntschaft, anzstud);
  lesenamen(name, anzstud);
  lesebekanntschaft(bekanntschaft, anzstud);
  matrixbewertung(bekanntschaft, anzstud);
  matrixausgabe(bekanntschaft, anzstud);
END stube.