MODULE integral;
(*******************************************************************
   Berechnung der Sinusfunktion mit Hilfe der Trapezregel.
   Die Schrittweite der Berechnungen mu explizit eingegeben werden.
 *******************************************************************)
FROM InOut    IMPORT WriteLn,
                     WriteString,
                     WriteCard,
                     ReadCard;
FROM IO       IMPORT WrReal;      (* Komfortableres WriteReal *)
FROM MathLib0 IMPORT sin;


CONST Pi = 3.141592654;
      A = 0.0;            (* untere Grenze *)
      B = Pi;             (* obere Grenze *)
VAR
  n : CARDINAL;   (* enthaelt die Anzahl der Teilintervalle *)

(*******************************************************************)
(* Procedure LESENAT                                               *)
(*******************************************************************)
(* Gibt die Nachricht m aus, liest die natuerliche Zahl zahl ein   *)
(* und gibt sie an das aufrufende Programm zurueck.                *)
(*******************************************************************)
PROCEDURE lesenat(m : ARRAY OF CHAR) : CARDINAL;
VAR
  zahl : CARDINAL;

BEGIN
  REPEAT
    WriteString(m);   (* Nachricht ausgeben *)
    ReadCard(zahl);   (* zahl einlesen *)
    WriteLn;
  UNTIL (zahl>0);     (* bis zahl>0 *)
  RETURN zahl;        (* mit zahl zureuck ans aufrufende Prg. *)
END lesenat;



(*******************************************************************)
(* Procedure TRAPEZREGEL                                           *)
(*******************************************************************)
(* Berechnet das Integral der Sinusfunktion von unteregrenze bis   *)
(* oberegrenze mittels der Trapezregel.                            *)
(*******************************************************************)
PROCEDURE trapezregel(unteregrenze,
                      oberegrenze : REAL;
                      schritt : CARDINAL) : REAL;
VAR
  schrittweite,
  summe,
  x1, x2 : REAL;    (* Laufvariablen *)

BEGIN
  summe:=FLOAT(0);  (* setzt summe wirklich gleich 0 *)
  schrittweite:=(oberegrenze-unteregrenze)/FLOAT(schritt);
  x1:=unteregrenze;
  WHILE (x1<oberegrenze) DO
    x2:=x1+schrittweite;
    summe:=summe+((sin(x1)+sin(x2))/2.0*schrittweite);
    x1:=x1+schrittweite;
  END;
  RETURN summe;
END trapezregel;


(*******************************************************************)
(* Hauptprogramm                                                   *)
(*******************************************************************)
BEGIN
  WriteLn;   (* Nachricht ausgeben *)
  WriteString('Dieses Programm berechnet das Integral '); WriteLn;
  WriteString('     '); WriteLn;
  WriteString('     '); WriteLn;
  WriteString('      sin(x) dx'); WriteLn;
  WriteString('     '); WriteLn;
  WriteString('     0'); WriteLn;
  WriteString('mittels der Trapezregel.');
  WriteLn; WriteLn;
  n:=lesenat('Bitte Anzahl der Schritte (< 65535) eingeben:  ');
  WriteString('Die Flaeche unter dem Graphen betraegt......: ');
  WrReal(trapezregel(A, B, n),8,10);
  WriteLn; WriteLn;
  WriteString('Ende der Berechnung.');
  WriteLn; WriteLn;
END integral.