IMPLEMENTATION MODULE FileIO;
(* Spike's Own Version of File Input/Output for Modula-2/86
   Copyright (C) 1983, 1984 Hugh J McLarty

   17 May 84	WriteChar changed to expand EOLchar
*)

IMPORT SYSTEM;
FROM SYSTEM IMPORT
  SWI, SETREG, GETREG, BYTE, WORD, ADDRESS, ADR, CODE;
IMPORT String;

CONST
  AX = 0; CX = 1; DX = 2; BX = 3; SI = 4; DI = 5; ES = 6;
  PushAX = 50H; PopDS = 1FH;
  DOSinterrupt = 21H;

TYPE
  File = CARDINAL;

VAR

  CRLF: ARRAY [0..1] OF CHAR;
  register: RECORD
	      DS: CARDINAL;
	      CASE tag: BOOLEAN OF
		FALSE: AX, BX, CX, DX: CARDINAL |
		TRUE:  AL,AH, BL,BH, CL,CH, DL,DH: BYTE
	      END
	    END;
  ErrorDecode: ARRAY [0..18] OF IOResultType;
  FileSpec: ARRAY [0..64] OF CHAR;


PROCEDURE DOS(VAR result: IOResultType);
  VAR error: CARDINAL;
  BEGIN
    error := 0;
    SETREG(AX,register.DS);
    CODE(PushAX);
    SETREG(AX,register.AX);
    SETREG(BX,register.BX);
    SETREG(CX,register.CX);
    SETREG(DX,register.DX);
    CODE(PopDS);
    SWI(DOSinterrupt);		(*	INT 21H		*)
    CODE(73H); CODE(3);		(* 	JNB NoErr	*)
    GETREG(AX,error);		(* 	MOV error,AX	*)
				(* NoErr:		*)
    GETREG(AX,register.AX);
    GETREG(BX,register.BX);
    GETREG(CX,register.CX);
    GETREG(DX,register.DX);
    result := ErrorDecode[error];
  END DOS;

PROCEDURE SetDSDX(add: ADDRESS);
  VAR  hack: RECORD
	      CASE tag: BOOLEAN OF
	        FALSE: address: ADDRESS |
	        TRUE:  offset, segment: CARDINAL
	      END (* CASE *)
	    END; (* RECORD *)
  BEGIN
    hack.address := add;
    register.DS := hack.segment;
    register.DX := hack.offset;
  END SetDSDX;

PROCEDURE Open(	VAR file:   File;
		    name:   ARRAY OF CHAR;
		    mode:   IOModeType;
		VAR result: IOResultType);
  BEGIN
    file := 0;
    String.Assign(FileSpec,name);
    SetDSDX(ADR(FileSpec));
    register.AL := BYTE(mode);	(* happens to work out...	*)
    register.AH := BYTE(3DH);
    DOS(result);
    IF result = OK THEN file := register.AX END;
  END Open;


PROCEDURE Create(VAR file:   File;
		     name:   ARRAY OF CHAR;
		     mode:   IOModeType;
		     erase:  BOOLEAN;
		 VAR result: IOResultType);
  BEGIN
    String.Assign(FileSpec,name);
    IF NOT erase THEN
      Open(file,name,readmode,result);
      IF result <> NameError THEN	(* if file might exist..	*)
        Close(file,result);
        result := NameError;
        RETURN;
      END;
    END;
    file := 0;
    SetDSDX(ADR(FileSpec));
    register.CX := 0;
    register.AH := BYTE(3CH);
    DOS(result);
    IF result = OK THEN file := register.AX END;
  END Create;


PROCEDURE Close(VAR file: File; VAR result: IOResultType);
  BEGIN
    register.BX := file;
    register.AH := BYTE(3EH);
    DOS(result);
    file := 0;
  END Close;

PROCEDURE GetPos(VAR file: File; VAR pos: FilePos; VAR result: IOResultType);
BEGIN
  register.BX := file;
  register.CX := 0;
  register.DX := 0;
  register.AL := BYTE(1);
  register.AH := BYTE(42H);
  DOS(result);
  pos.high := register.DX;
  pos.low  := register.AX;
END GetPos;

PROCEDURE SetPos(VAR file: File; pos: FilePos; VAR result: IOResultType);
BEGIN
  register.BX := file;
  register.CX := pos.high;
  register.DX := pos.low;
  register.AL := BYTE(0);
  register.AH := BYTE(42H);
  DOS(result);
END SetPos;

PROCEDURE ReadNBytes(VAR file:   File;
		     VAR data:   ARRAY OF BYTE;
		         nbytes: CARDINAL;
		     VAR count:  CARDINAL;
		     VAR result: IOResultType);
  BEGIN
    count := 0;
    SetDSDX(ADR(data));
    register.CX := nbytes;
    register.BX := file;
    register.AH := BYTE(3FH);
    DOS(result);
    IF result = OK THEN count := register.AX END;
  END ReadNBytes;


PROCEDURE ReadChar(VAR file:   File;
		   VAR data:   CHAR;
		   VAR result: IOResultType);
  VAR count: CARDINAL;
  BEGIN
    ReadNBytes(file,data,1,count,result);
    IF (result = OK) AND (count = 0) THEN
      result := EofError;
    END
  END ReadChar;

PROCEDURE ReadWord(VAR file:   File;
		   VAR data:   WORD;
		   VAR result: IOResultType);
  VAR count: CARDINAL;
  BEGIN
    ReadNBytes(file,data,2,count,result);
    IF (result = OK) AND (count < 2) THEN
      result := EofError;
    END
  END ReadWord;


PROCEDURE WriteChar(VAR file:   File;
			data:   CHAR;
		    VAR result: IOResultType);
  BEGIN
    IF data = EOLChar THEN
      WriteNBytes(file,CRLF,2,result)
    ELSE
      WriteNBytes(file,data,1,result);
    END;
  END WriteChar;

PROCEDURE WriteNBytes(VAR file:   File;
		      VAR data:   ARRAY OF BYTE;
			  nbytes: CARDINAL;
		      VAR result: IOResultType);
  BEGIN
    SetDSDX(ADR(data));
    register.CX := nbytes;
    register.BX := file;
    register.AH := BYTE(40H);
    DOS(result);
    IF (result = OK) AND (register.AX <> nbytes) THEN
      result := SpaceError;
    END;
  END WriteNBytes;

PROCEDURE WriteWord(VAR file:   File;
			data:   WORD;
		    VAR result: IOResultType);
  BEGIN
    WriteNBytes(file,data,2,result);
  END WriteWord;


PROCEDURE WriteString(VAR file:   File;
			  data:   ARRAY OF CHAR;
		      VAR result: IOResultType);
  BEGIN
    WriteNBytes(file,data,String.Length(data),result);
  END WriteString;


BEGIN
  CRLF[0] := 15C; CRLF[1] := 12C;
  ErrorDecode[0] := OK;
  ErrorDecode[1] := UsageError;		(* Invalid function number	*)
  ErrorDecode[2] := NameError;		(* File not found		*)
  ErrorDecode[3] := NameError;		(* Path not found		*)
  ErrorDecode[4] := UsageError;		(* too many open files		*)
  ErrorDecode[5] := AccessError;	(* Access denied		*)
  ErrorDecode[6] := StatusError;	(* Invalid handle		*)
  ErrorDecode[12]:= AccessError;	(* Invalid access code		*)
  ErrorDecode[15]:= UsageError;		(* Invalid drive specified	*)
END FileIO.
