IMPLEMENTATION MODULE FindParameter;

FROM SYSTEM IMPORT SWI, RTSVECTOR, SETREG, GETREG,
                   AX, BX, CX;
FROM ASCII IMPORT nul;

TYPE 
    PSP = RECORD
              stuff: ARRAY[1..128] OF CHAR;
              commTail: ARRAY[0..127] OF CHAR;
    END;

VAR
    PSPPtr: RECORD
               CASE BOOLEAN OF
                 TRUE: addr: POINTER TO PSP;
               | FALSE: offset, base: CARDINAL;
               END;
            END;
    LastParam,      (*index to the begin of the last processed parameter*)
    CurrParam,      (*index to the begin of the following parameter*)
    parambegin,     (*indicates the begin of the parameter in commmTail*)
    tailc       : CARDINAL;
    firstuse    : BOOLEAN;       (*true: you have to read the PSP*)

PROCEDURE FNChar (ch: CHAR) : BOOLEAN;
BEGIN
  CASE ch OF
    'A'..'Z', '0'..'9', 'a'..'z',
    '$', '&', '#', '@', '!', '%',
    "'", '`', '(', ')', '-', '_',
    '^', '~', '.', ':', '\' :RETURN (TRUE);
  ELSE RETURN (FALSE);
  END;
END FNChar;


PROCEDURE FetchParameter(Element: Order; VAR Parameter: ARRAY OF CHAR): Result;

VAR
    i      : CARDINAL;
    Status : Result;

BEGIN
  (*
   DOS puts the command string in the Program Segment Prefix (PSP).
   This module is reading the command string and fill in Parameter
   the with ElementOrder spezified Parameter.
  *)
  Status := OK;
  IF firstuse THEN
    firstuse := FALSE;
    (*
    We need first the PSP from RTS:
    *)
    SETREG(AX,0026H);  (*RTS(38) get Program Segment Prefix Pointer *)
    SWI(RTSVECTOR);    (*RTS call*)
    GETREG(BX,PSPPtr.offset);
    GETREG(CX,PSPPtr.base);
    (*
    Get length of command string and move it to tailc 
    *)
    tailc := ORD(PSPPtr.addr^.commTail[0]);       
    (*
    the command string has the following structure:
      1: rtsname (not in PSP, read and skipped by DOS)
      2: parameters (for RTS, optional)
      3: separator (one or more spaces)
      4: M-2-program-name
      5: parameters (for M-2 programs, optional)
    We have to skip now 1 - 4:
    *)
    CurrParam:= 1;
    WHILE (CurrParam <= tailc) AND (PSPPtr.addr^.commTail[CurrParam] <> ' ') DO
      INC(CurrParam);          (*skip leading parameters*)
    END;
    WHILE (CurrParam <= tailc) AND (PSPPtr.addr^.commTail[CurrParam] = ' ') DO
      INC(CurrParam);          (*skip leading blanks*)
    END;             
    WHILE (CurrParam <= tailc) AND (FNChar(PSPPtr.addr^.commTail[CurrParam])) DO
      INC(CurrParam);          (*skip the program name*)
    END;
    WHILE (CurrParam <= tailc) AND (PSPPtr.addr^.commTail[CurrParam] = ' ') DO
      INC(CurrParam);          (*skip seperator immediatly after progname*)
    END;
    parambegin := CurrParam;
  END;
  (*
    depending on Element, we have to combine the Last- and CurrParam
  *)
  IF Element = First THEN
    CurrParam := parambegin;
    LastParam := parambegin;
  ELSIF (Element = Next) AND (CurrParam <= tailc) THEN
    LastParam := CurrParam;
  ELSIF Element = Same THEN
    CurrParam := LastParam;
  ELSIF (LastParam > parambegin) AND (Element = Previous) THEN  
  (*
    To find the predecessor, you have first to skip the last
    two parameters and the blanks betweeen them
  *)
    DEC(CurrParam);
    WHILE PSPPtr.addr^.commTail[CurrParam] = ' ' DO
      DEC(CurrParam);
    END;
    (*
       skip the last parameter
    *)
    WHILE PSPPtr.addr^.commTail[CurrParam] <> ' ' DO
      DEC(CurrParam);
    END;
    WHILE PSPPtr.addr^.commTail[CurrParam] = ' ' DO
      DEC(CurrParam);
    END;
    (*
      skip the last but one parameter
    *)
    WHILE PSPPtr.addr^.commTail[CurrParam] <> ' ' DO
      DEC(CurrParam);
    END;
    INC(CurrParam);
  ELSE 
    Status := NotFound;
  END;
  (* 
    CurrParam indicates now the beginning of the next parameter, the
    end or before the beginning of the command string parameter
  *)
  IF CurrParam < parambegin THEN    (*there exist no preceding parameter*)
    Status := NotFound;
    CurrParam := parambegin;
    LastParam := parambegin;
    ELSIF CurrParam > tailc THEN      (*there exist no following parameter*)
    Status := NotFound;    
  ELSIF Status = OK THEN
    LastParam := CurrParam;
    WHILE (PSPPtr.addr^.commTail[CurrParam] <> ' ') AND (CurrParam <= tailc) DO
      INC(CurrParam);
    END;
  END;
  (*
    we have now to look, if there is in Parameter enough space for the 
    actuel selected parameter
  *)
  IF ((CurrParam - LastParam) > (HIGH(Parameter) + 1)) AND
     (Status = OK) THEN
    Status := InsuffSpace;
  END;
  (*
    if there is enough space and there is another parameter:
    write the actuel parameter in Parameter and termiate it with nul
  *)
  IF Status = OK THEN
    CurrParam := LastParam;
    i := 0;
    WHILE (PSPPtr.addr^.commTail[CurrParam] <> ' ') AND
          (CurrParam <= tailc) DO
      Parameter[i] := PSPPtr.addr^.commTail[CurrParam];
      INC(CurrParam);
      INC(i);
    END;
    (*
       add nul if there is enough space
    *)
    IF i < (HIGH(Parameter) + 1) THEN
      Parameter[i] := nul;
    END;
  END;
  (*
    skip blanks between parameters
  *)
  WHILE (PSPPtr.addr^.commTail[CurrParam] = ' ') AND
        (CurrParam <= tailc) DO
    INC(CurrParam);
  END;
  RETURN (Status);
END FetchParameter;

BEGIN
  firstuse := TRUE;
END FindParameter.

